require 'books'
require 'book'

RSpec.describe Books do
  before do
    @books = Books.new
    @sholohov_book = Book.new('Sholohov', 'Povest o nastojashem cheloveke', 'Povest', 500)
    @sholohov_book_two = Book.new('Sholohov', 'Tihij don', 'Roman', 1500)
    @dostoevsky_book = Book.new('Dostoevsky', 'Prestuplenije i nakazanije', 'Roman', 1000)
  end

  context '#each' do
    it 'should provide books sorted by author' do
      @books.add(@sholohov_book)
      @books.add(@dostoevsky_book)
      authors = []
      @books.each do |book|
        authors.append(book.author)
      end
      expect(authors).to eq([@dostoevsky_book.author, @sholohov_book.author])
    end

    it 'should provide books sorted by name' do
      @books.add(@sholohov_book_two)
      @books.add(@sholohov_book)
      names = []
      @books.each do |book|
        names.append(book.name)
      end
      expect(names).to eq([@sholohov_book.name, @sholohov_book_two.name])
    end

    it 'should return enumerator when no block given' do
      expect(@books.each).to be_a(Enumerator)
    end
  end

  context do
    before do
      @books.add(@dostoevsky_book)
      @books.add(@sholohov_book)
      @books.add(@sholohov_book_two)
    end

    context '#books_by_genre' do
      it 'should return books corresponding to a genre' do
        expect(@books.books_by_genre('Roman')).to contain_exactly(@sholohov_book_two,
                                                                  @dostoevsky_book)
      end

      it 'should return empty book list if genre is not found' do
        expect(@books.books_by_genre('Fiction')).to be_empty
      end
    end

    context '#genre' do
      it 'should return a list of known genres' do
        expect(@books.genres).to contain_exactly(@dostoevsky_book.genre,
                                                 @sholohov_book.genre)
      end
    end

    context '#by_number' do
      it 'should return book by the number' do
        expect(@books.by_number(1)).to eq(@sholohov_book)
      end
    end
  end
end
